package com.convertlab.dmhubsdk;

import android.util.Log;

import androidx.annotation.NonNull;

import com.convertlab.dmhubsdk.models.DMHubGeTui;
import com.convertlab.dmhubsdk.models.DMHubIdentity;
import com.convertlab.dmhubsdk.models.DMHubOptions;
import com.convertlab.dmhubsdk.models.DMHubTrackMap;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.Result;

public class DMHubClientWrapper extends DMHubWrapper implements MethodChannel.MethodCallHandler {
    static DMHubClientWrapper wrapper;

    DMHubClientWrapper(FlutterPlugin.FlutterPluginBinding flutterPluginBinding, String channelName) {
        super(flutterPluginBinding, channelName);
        wrapper = this;
    }
    public static DMHubClientWrapper getInstance() {
        return wrapper;
    }

    @Override
    public void onMethodCall(MethodCall call, @NonNull MethodChannel.Result result) {
        JSONObject param = (JSONObject) call.arguments;
        Log.d("macros|MethodCall>param",param.toString());
        try{
            if (DMHubSDKMethod.init.equals(call.method)) {
                init(param, DMHubSDKMethod.init, result);
            }else if(DMHubSDKMethod.setIdentity.equals(call.method)){
                setIdentity(param, DMHubSDKMethod.init, result);
            }else if(DMHubSDKMethod.currentIdentity.equals(call.method)){
                currentIdentity(DMHubSDKMethod.currentIdentity, result);
            }else if(DMHubSDKMethod.clearIdentity.equals(call.method)){
                clearIdentity(DMHubSDKMethod.clearIdentity, result);
            }else if(DMHubSDKMethod.trackMap.equals(call.method)){
                trackMap(param, DMHubSDKMethod.trackMap, result);
            }else if(DMHubSDKMethod.track.equals(call.method)){
                track(param, DMHubSDKMethod.track, result);
            }else if(DMHubSDKMethod.trackOpenView.equals(call.method)){
                trackOpenView(param.getString("viewName"), DMHubSDKMethod.trackOpenView, result);
            }else if(DMHubSDKMethod.trackExitView.equals(call.method)){
                trackExitView(param.getString("viewName"), DMHubSDKMethod.trackExitView, result);
            }else if(DMHubSDKMethod.onJPushReceiveBroadcast.equals(call.method)){
                onJPushReceiveBroadcast(DMHubSDKMethod.onJPushReceiveBroadcast,result);
            }else if(DMHubSDKMethod.onJPushReceiveRegisterId.equals(call.method)){
                onJPushReceiveRegisterId(param,DMHubSDKMethod.onJPushReceiveRegisterId,result);
            }else if(DMHubSDKMethod.onGeTuiReceiveClientId.equals(call.method)){
                onGeTuiReceiveClientId(param,DMHubSDKMethod.onGeTuiReceiveClientId,result);
            }else if(DMHubSDKMethod.onJPushReceiveMessageData.equals(call.method)){
                onJPushReceiveMessageData(param,DMHubSDKMethod.onJPushReceiveMessageData, result);
            }else if(DMHubSDKMethod.onJPushNotificationMessageArrived.equals(call.method)){
                onJPushNotificationMessageArrived(param,DMHubSDKMethod.onJPushNotificationMessageArrived, result);
            }else if(DMHubSDKMethod.onGeTuiReceiveMessageData.equals(call.method)){
                onGeTuiReceiveMessageData(param,DMHubSDKMethod.onGeTuiReceiveMessageData, result);
            }else if(DMHubSDKMethod.onGeTuiNotificationMessageArrived.equals(call.method)){
                onGeTuiNotificationMessageArrived(param,DMHubSDKMethod.onGeTuiNotificationMessageArrived, result);
            }else if(DMHubSDKMethod.onGlobalParamsPush.equals(call.method)){
                onGlobalParamsPush(param,DMHubSDKMethod.onGlobalParamsPush, result);
            }
        }catch (Exception ignored) {}
    }

    /**
     * 在自定义的 Application 中初始化 DMHubSDK
     * 在整个应用程序全局，只需要进行一次初始化
     */
    private void init(JSONObject param, String channelName, Result result) throws JSONException {
        DMHubOptions options = DMHubOptionHelper.fromJson(param);
        // 初始化 DMHubSDK
        DMHubSDK.init(this.mContext, Configuration.create(
                options.getTrackUrl(),
                options.getAppId(),
                options.getAppName()
        ).enableDebugLogging(options.isEnableDebugLogging()).setAutoTrackOpenAppEvent(options.isAutoTrackOpenAppEvent()));

        Map<String, Object> data = new HashMap<>();
        onSuccess(result, channelName, data);
    }

    /**
     * 设置客户身份，用于绑定事件
     * 如果重复设置，将会使用新的客户身份进行事件绑定
     */
    private void setIdentity(JSONObject param, String channelName, Result result) {
        asyncRunnable(()->{
            try {
                DMHubIdentity options = DMHubIdentityHelper.fromJson(param);
                DMHubSDK.setIdentity(options.getIdentityType(),options.getIdentityValue());

                Map<String, Object> data = new HashMap<>();
                data.put("result","success");
                onSuccess(result, channelName, data);
            } catch (JSONException e) {
                onError(result, e);
            }
        });
    }
    /**
     * 获取当前设置的客户身份
     * @return 如果当前还未设置过有效的客户身份，返回 null
     */
    private void currentIdentity(String channelName, Result result){
        asyncRunnable(()->{
            DMHubSDK.DMHubIdentity dmHubIdentity = DMHubSDK.currentIdentity();
            Map<String, Object> data = new HashMap<>();
            data.put("type",dmHubIdentity!=null?dmHubIdentity.getType():"");
            data.put("value",dmHubIdentity!=null?dmHubIdentity.getValue():"");
            onSuccess(result, channelName, data);
        });
    }
    /**
     * 清除当前设置的客户身份
     * @return 当前设置的客户身份，如果当前还未设置过有效的客户身份，返回 null
     */
    private void clearIdentity(String channelName, Result result){
        asyncRunnable(()->{
            DMHubSDK.DMHubIdentity dmHubIdentity = DMHubSDK.clearIdentity();
            Map<String, Object> data = new HashMap<>();
            data.put("type",dmHubIdentity!=null?dmHubIdentity.getType():"");
            data.put("value",dmHubIdentity!=null?dmHubIdentity.getValue():"");
            onSuccess(result, channelName, data);
        });
    }
    /**
     * 跟踪自定义客户事件
     * eventId    在 DM Hub 中新建的自定义事件的事件 Id
     * properties 事件的自定义属性，必须以在 DM Hub 中新建自定义事件时添加的自定义属性作为 key
     */
    private void trackMap(JSONObject param, String channelName, Result result) {
        asyncRunnable(()->{
            try {
                DMHubTrackMap options = DMHubTrackMapHelper.trackMapFromJson(param);
                DMHubSDK.trackMap(options.getEventId(),options.getProperties());

                Map<String, Object> data = new HashMap<>();
                onSuccess(result, channelName, data);
            } catch (Exception e) {
                onError(result, e);
            }
        });
    }
    /**
     * 跟踪自定义客户事件
     * eventId    在 DM Hub 中新建的自定义事件的事件 Id
     */
    private void track(JSONObject param, String channelName, Result result) {
        asyncRunnable(()->{
            try {
                DMHubTrackMap options = DMHubTrackMapHelper.fromJson(param);
                DMHubSDK.track(options.getEventId());

                Map<String, Object> data = new HashMap<>();
                onSuccess(result, channelName, data);
            } catch (Exception e) {
                onError(result, e);
            }
        });
    }
    /**
     * 跟踪进入视图事件
     *
     * viewName 视图的名称，客户时间轴上的显示为：'进入手机视图 ${viewName}'
     */
    private void trackOpenView(String viewName, String channelName, Result result) {
        asyncRunnable(()->{
            DMHubSDK.trackOpenView(viewName);
            Map<String, Object> data = new HashMap<>();
            onSuccess(result, channelName, data);
        });
    }
    /**
     * 跟踪离开视图事件
     *
     * viewName 视图的名称，客户时间轴上的显示为：'离开手机视图 ${viewName}'
     */
    private void trackExitView(String viewName, String channelName, Result result){
        asyncRunnable(()->{
            DMHubSDK.trackExitView(viewName);

            Map<String, Object> data = new HashMap<>();
            onSuccess(result, channelName, data);
        });
    }
    /**
     * 当自定义的 JPush 广播接收器收到来自 JPush 的广播时调用
     *
     * intent 收到的广播的 intent
     */
    private void onJPushReceiveBroadcast(String channelName, Result result){
        asyncRunnable(()->{
            DMHubSDK.onJPushReceiveBroadcast("dmhubsdk",this.mIntent);
            Map<String, Object> data = new HashMap<>();
            onSuccess(result, channelName, data);
        });
    }

    /**
     * 在自定义的 GTIntentService 的 onJPushReceiveRegisterId 方法中调用
     * @param jsonObj 传入的JSON数据对象
     */
    private void onJPushReceiveRegisterId(JSONObject jsonObj, String channelName, Result result){
        asyncRunnable(()->{
            try {
                DMHubSDK.onJPushReceiveRegisterId(jsonObj.getString("appKey"),jsonObj.getString("clientId"));
            } catch (JSONException e) {
                e.printStackTrace();
            }
            Map<String, Object> data = new HashMap<>();
            onSuccess(result, channelName, data);
        });
    }

    /**
     *  在自定义的 GTIntentService 的 onReceiveClientId 方法中调用
     *  @param  jsonObj 传入的JSON数据对象
     */
    private void onGeTuiReceiveClientId(JSONObject jsonObj, String channelName, Result result) {
        asyncRunnable(()->{
            try {
                DMHubSDK.onGeTuiReceiveClientId(jsonObj.getString("appKey"),jsonObj.getString("clientId"));
            } catch (JSONException e) {
                e.printStackTrace();
            }
            Map<String, Object> data = new HashMap<>();
            onSuccess(result, channelName, data);
        });
    }

    /**
    * 在自定义的 GTIntentService 的 onReceiveMessageData 方法中调用
    *
    * taskId    GTTransmitMessage 中携带的 taskId
    * messageId GTTransmitMessage 中携带的 messageId
    * payload   GTTransmitMessage 中携带的 payload
    */
    private void onJPushReceiveMessageData(JSONObject jsonObj, String channelName, Result result){
        asyncRunnable(()->{
            try {
                Map<String, Object> bundleMap = new HashMap<>();
                //循环转换
                Iterator<String> it = jsonObj.keys();
                while (it.hasNext()) {
                    String key = String.valueOf(it.next());
                    String value = (String) jsonObj.get(key);
                    bundleMap.put(key,value);
                }
                DMHubSDK.onJPushReceiveMessageData(bundleMap);
                Map<String, Object> returnData = new HashMap<>();
                onSuccess(result, channelName, returnData);
            } catch (JSONException e) {
                onError(result, e);
            }
        });
    }

    /**
     * 在自定义的 GTIntentService 的 onReceiveMessageData 方法中调用
     *
     * taskId    GTTransmitMessage 中携带的 taskId
     * messageId GTTransmitMessage 中携带的 messageId
     * payload   GTTransmitMessage 中携带的 payload
     */
    private void onJPushNotificationMessageArrived(JSONObject jsonObj, String channelName, Result result){
        asyncRunnable(()->{
            try {
                Map<String, Object> bundleMap = new HashMap<>();
                //循环转换
                Iterator<String> it = jsonObj.keys();
                while (it.hasNext()) {
                    String key = String.valueOf(it.next());
                    String value = (String) jsonObj.get(key);
                    bundleMap.put(key,value);
                }
                DMHubSDK.onJPushNotificationMessageArrived(bundleMap);
                Map<String, Object> returnData = new HashMap<>();
                onSuccess(result, channelName, returnData);
            } catch (JSONException e) {
                onError(result, e);
            }
        });
    }

    /**
     * 在自定义的 GTIntentService 的 onReceiveMessageData 方法中调用
     *
     * taskId    GTTransmitMessage 中携带的 taskId
     * messageId GTTransmitMessage 中携带的 messageId
     * payload   GTTransmitMessage 中携带的 payload
     */
    private void onGeTuiReceiveMessageData(JSONObject param, String channelName, Result result){
        asyncRunnable(()->{
            try {
                DMHubGeTui options = DMHubGeTuiHelper.fromJson(param);
                DMHubSDK.onGeTuiReceiveMessageData(options.getTaskId(),options.getMessageId(),options.getPlayload());
                Map<String, Object> data = new HashMap<>();
                onSuccess(result, channelName, data);
            } catch (JSONException e) {
                onError(result, e);
            }
        });
    }
    /**
     * 在自定义的 GTIntentService 的 onNotificationMessageArrived 方法中调用
     *
     * taskId    GTNotificationMessage 中携带的 taskId
     * messageId GTNotificationMessage 中携带的 messageId
     */
    private void onGeTuiNotificationMessageArrived(JSONObject param, String channelName, Result result){
        asyncRunnable(()->{
            try {
                DMHubGeTui options = DMHubGeTuiHelper.fromJson(param);
                DMHubSDK.onGeTuiNotificationMessageArrived(options.getTaskId(),options.getMessageId());
                Map<String, Object> data = new HashMap<>();
                onSuccess(result, channelName, data);
            } catch (JSONException e) {
                onError(result, e);
            }
        });
    }

    /**
     * 如果一个应用设置的追踪代码较多，而且他们之间有一些公共的参数，如果在每个追踪点都分别设置这些参数，会给代码维护带来不便。
     * 为此，DMHubSDK引入了push方法，用来设置应用全局的参数。通过push方法设置的参数，在该应用后续的track方法调用时，会自动传递。
     * @param jsonObject
     * @param channelName
     * @param result
     * @throws JSONException
     */
    private void onGlobalParamsPush(JSONObject jsonObject,String channelName, Result result) throws JSONException{
        //循环转换
        Iterator<String> it = jsonObject.keys();
        while (it.hasNext()) {
            String key = it.next();
            Object value = jsonObject.get(key);
            DMHubSDK.push(key,value);
//            Log.d("macros|paramsPush==",key+","+value);
        }
        Map<String, Object> data = new HashMap<>();
        onSuccess(result, channelName, data);
    }
}
