package com.convertlab.dmhubsdk;

import android.util.Log;

import com.convertlab.dmhubsdk.models.DMHubGeTui;
import com.convertlab.dmhubsdk.models.DMHubIdentity;
import com.convertlab.dmhubsdk.models.DMHubOptions;
import com.convertlab.dmhubsdk.models.DMHubTrackMap;

import org.json.JSONException;
import org.json.JSONObject;

import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

class DMHubOptionHelper {
    static DMHubOptions fromJson(JSONObject json) throws JSONException {
        DMHubOptions options = new DMHubOptions();
        options.setTrackUrl(json.getString("trackUrl"));
        options.setAppId(json.getString("appId"));
        options.setAppName(json.getString("appName"));
        options.setEnableDebugLogging(json.getBoolean("enableDebugLogging"));
        options.setAutoTrackOpenAppEvent(json.getBoolean("autoTrackOpenAppEvent"));
        return options;
    }

    static Map<String, Object> toJson(DMHubOptions options) {
        Map<String, Object> data = new HashMap<>();
        data.put("trackUrl", options.getTrackUrl());
        data.put("appId", options.getAppId());
        data.put("appName", options.getAppName());
        data.put("enableDebugLogging", options.isEnableDebugLogging());
        data.put("autoTrackOpenAppEvent", options.isAutoTrackOpenAppEvent());
        return data;
    }
}
class DMHubIdentityHelper{
    static DMHubIdentity fromJson(JSONObject json) throws JSONException {
        DMHubIdentity options = new DMHubIdentity();
        options.setIdentityType(json.getString("identityType"));
        options.setIdentityValue(json.getString("identityValue"));
        return options;
    }

    static Map<String, Object> toJson(DMHubSDK.DMHubIdentity options) {
        Map<String, Object> data = new HashMap<>();
        data.put("identityType", options.getType());
        data.put("identityValue", options.getValue());
        return data;
    }
}
class DMHubTrackMapHelper{
    static DMHubTrackMap trackMapFromJson(JSONObject jsonObject) throws JSONException {
        DMHubTrackMap trackMap = new DMHubTrackMap();
        trackMap.setEventId(jsonObject.getString("eventId"));

        JSONObject mapProp = jsonObject.optJSONObject("properties");

        Map<String, Object> data =new HashMap<>();
        //循环转换
        if(mapProp != null) {
            Log.d("DMHubTrackMapHelper","mapProp对象:" + mapProp.toString());

            Iterator<String> it = mapProp.keys();
            while (it.hasNext()) {
                String key = it.next();
                Object value = mapProp.opt(key);
                if(value != null) {
                    data.put(key, value);
                    Log.d("macros|trackMap==",key+","+value);
                }

            }
        }

        Log.d("DMHubTrackMapHelper","map对象:"+data.toString());
        trackMap.setProperties(data);
        return trackMap;
    }
    static DMHubTrackMap fromJson(JSONObject json) throws JSONException {
        DMHubTrackMap trackMap = new DMHubTrackMap();
        trackMap.setEventId(json.getString("eventId"));
        return trackMap;
    }

    static Map<String, Object> toJson(DMHubTrackMap options) {
        Map<String, Object> data = new HashMap<>();
        data.put("eventId", options.getEventId());
        data.put("properties", options.getProperties());
        return data;
    }
}
class DMHubGeTuiHelper{
    static DMHubGeTui fromJson(JSONObject json) throws JSONException {
        DMHubGeTui options = new DMHubGeTui();
        options.setTaskId(json.getString("taskId"));
        options.setMessageId(json.getString("messageId"));
        options.setPlayload(json.get("playload").toString().getBytes(StandardCharsets.UTF_8));
        return options;
    }

    static Map<String, Object> toJson(DMHubGeTui options) {
        Map<String, Object> data = new HashMap<>();
        data.put("taskId", options.getTaskId());
        data.put("messageId", options.getMessageId());
        data.put("payload",options.getPlayload());
        return data;
    }
}