package com.convertlab.dmhubsdk;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.util.Log;

import androidx.annotation.NonNull;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.JSONMethodCodec;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;

public class DMHubWrapper implements ActivityAware, MethodChannel.MethodCallHandler {
    private final String TAG = this.getClass().getName();
    private static final String CHANNEL_PREFIX = "com.convertlab.dmhub/";
    private final ExecutorService cachedThreadPool = Executors.newCachedThreadPool();

    public Context mContext;
    private Activity mActivity;
    public Intent mIntent;
    private Application mApplication;
    public MethodChannel channel;

    public DMHubWrapper(FlutterPlugin.FlutterPluginBinding flutterPluginBinding, String channelName) {
        this.mContext = flutterPluginBinding.getApplicationContext();
        this.channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), CHANNEL_PREFIX + channelName, JSONMethodCodec.INSTANCE);
        channel.setMethodCallHandler(this);
    }

    public void post(Runnable runnable) {
        DmhubsdkPlugin.handler.post(runnable);
    }

    public void asyncRunnable(Runnable runnable) {
        cachedThreadPool.execute(runnable);
    }

    public void onSuccess(MethodChannel.Result result, String channelName, Object object) {
        post(()-> {
            Map<String, Object> data = new HashMap<>();
            if (object != null) {
                data.put(channelName, object);
            }
            result.success(data);
        });
    }

    public void onError(MethodChannel.Result result, Exception e) {
        post(()-> {
            Map<String, Object> data = new HashMap<>();
            data.put("error", e.getMessage());
            result.success(data);
        });
    }

    @Override
    public void onMethodCall(MethodCall call, MethodChannel.Result result) {
        result.notImplemented();
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        Log.d(TAG,"onAttachedToActivity method");
        mContext = binding.getActivity().getApplicationContext();
        mActivity = binding.getActivity();
        mIntent = binding.getActivity().getIntent();
        mApplication = binding.getActivity().getApplication();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {
        mActivity = null;
    }
}
