//
//  DMHubClientWrapper.m
//  dmhubsdk
//
//  Created by macroswang on 2022/01/08.
//
#import <DMHubSDK/DMHubSDK.h>
#import "DMHubClientWrapper.h"
#import "DMHubSDKMethod.h"

@interface DMHubClientWrapper () <FlutterPlugin>
@end

@implementation DMHubClientWrapper

static DMHubClientWrapper *wrapper = nil;

+ (DMHubClientWrapper *)sharedWrapper {
    return wrapper;
}

- (void)sendDataToFlutter:(NSDictionary *)aData {
    if (aData == nil) {
        return;
    }
    [self.channel invokeMethod:DMHubSDKMethodKeySendDataToFlutter
                     arguments:aData];
}

+ (DMHubClientWrapper *)channelName:(NSString *)aChannelName
                       registrar:(NSObject<FlutterPluginRegistrar>*)registrar
{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        wrapper = [[DMHubClientWrapper alloc] initWithChannelName:aChannelName registrar:registrar];
    });
    return wrapper;
}

- (instancetype)initWithChannelName:(NSString *)aChannelName
                          registrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    if(self = [super initWithChannelName:aChannelName
                               registrar:registrar]) {
        [registrar addApplicationDelegate:self];
    }
    return self;
}

#pragma mark - FlutterPlugin
- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
#ifdef DEBUG
    NSLog(@"macroswang>>>>>>>handleMethodCall,%@",call.arguments);
#endif
    if ([DMHubSDKMethodKeyInit isEqualToString:call.method])
    {
        [self startWithUrl:call.arguments result:result];
    }
    else if ([DMHubSDKMethodKeyInitWithOptions isEqualToString:call.method])
    {
        [self startWithOptions:call.arguments result:result];
    }
    else if ([DMHubSDKSetIdentity isEqualToString:call.method])
    {
        [self setIdentity:call.arguments result:result];
    }
    else if ([DMHubSDKCurrentIdentity isEqualToString:call.method])
    {
        [self currentIdentity:call.arguments result:result];
    }
    else if ([DMHubSDKClearIdentity isEqualToString:call.method])
    {
        [self clearIdentity:call.arguments result:result];
    }
    else if ([DMHubSDKTrackMap isEqualToString:call.method])
    {
        [self trackMap:call.arguments result:result];
    }
    else if ([DMHubSDKTrackOpenView isEqualToString:call.method])
    {
        [self trackOpenView:call.arguments result:result];
    }
    else if ([DMHubSDKTrackExitView isEqualToString:call.method])
    {
        [self trackExitView:call.arguments result:result];
    }
    else if ([DMHubSDKAppKey isEqualToString:call.method])
    {
        [self setAppKey:call.arguments result:result];
    }
    else if ([DMHubSDKAppKeyWithPushId isEqualToString:call.method])
    {
        [self setAppKeyWithPushId:call.arguments result:result];
    }
    else if ([DMHubSDKTrackReceiveJPushMessage isEqualToString:call.method])
    {
        [self trackReceiveJPushMessage:call.arguments result:result];
    }
    else if ([DMHubSDKOnJPushReceiveMessageData isEqualToString:call.method])
    {
        [self onJPushReceiveMessageData:call.arguments result:result];
    }
    else if ([DMHubSDKOnJPushReceiveRegisterId isEqualToString:call.method])
    {
        [self onJPushReceiveRegisterId:call.arguments result:result];
    }
    else if ([DMHubSDKOnJPushNotificationMessageArrived isEqualToString:call.method])
    {
        [self onJPushNotificationMessageArrived:call.arguments result:result];
    }
    else if([DMHubSDKOnGeTuiReceiveClientId isEqualToString:call.method])
    {
        [self onGeTuiReceiveClientId:call.arguments result:result];
    }
    else if([DMHubSDKTrackReceiveGeTuiPayloadData isEqualToString:call.method])
    {
        [self trackReceiveGeTuiPayloadData:call.arguments result:result];
    }
    else if([DMHubSDKOnGlobalParamsPush isEqualToString:call.method])
    {
        [self onGlobalParamsPush:call.arguments result:result];
    }
    else {
        [super handleMethodCall:call result:result];
    }
}


#pragma mark - 接口列表
/**
* 启动 DMHubSDK
*  trackUrl 在 DM Hub 平台创建应用时获得的 url
*  appId 在 DM Hub 平台创建应用时获得的 appid
*  appName 在时间轴中显示的应用名
**/
- (void)startWithUrl:(NSDictionary *)param result:(FlutterResult)result {
    __weak typeof(self) weakSelf = self;
#ifdef DEBUG
    NSLog(@"macroswang>>>>>>>startWithUrl|trackUrl=%@,appid=%@,appName=%@",param[@"trackUrl"],param[@"appId"],param[@"appName"]);
#endif
    // 启动 DMHubSDK
    [DMHubSDK startWithUrl:param[@"trackUrl"]
                     appId:param[@"appId"]
                   appName:param[@"appName"]
             launchOptions:nil];

    [weakSelf wrapperCallBack:result
                  channelName:DMHubSDKMethodKeyInit
                       object:@{
                           @"appId": param[@"appId"],
                           @"appName": param[@"appName"],
                           @"trackUrl":param[@"trackUrl"]
                       }];
}
/**
* 启动 DMHubSDK
* param中包含 trackUrl 在 DM Hub 平台创建应用时获得的 url
* param中包含 appId 在 DM Hub 平台创建应用时获得的 appid
* param中包含 appName 在时间轴中显示的应用名
* param中包含 autoTrackViewEvent 是否自动记录进入、离开视图事件，默认值：NO
* param中包含 flushInterval 上传数据的时间间隔（单位：秒），默认值：30
* param中包含 launchOptions Application 的启动参数
**/
- (void)startWithOptions:(NSDictionary *)param result:(FlutterResult)result {
    __weak typeof(self) weakSelf = self;
    // 启动 DMHubSDK
    [DMHubSDK startWithUrl:param[@"trackUrl"]
                     appId:param[@"appId"]
                   appName:param[@"appName"]
        autoTrackViewEvent:[param[@"autoTrackOpenAppEvent"] boolValue]
             flushInterval:[param[@"flushInterval"] intValue]
             launchOptions:nil];

    [weakSelf wrapperCallBack:result
                  channelName:DMHubSDKMethodKeyInitWithOptions
                       object:@{
                           @"appId": param[@"appId"],
                           @"appName": param[@"appName"],
                           @"trackUrl":param[@"trackUrl"]
                       }];
}
/**
 * 设置客户身份，用于绑定事件
 * 如果重复设置，将会使用新的客户身份进行事件绑定
 */
- (void)setIdentity:(NSDictionary *)param result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;
    NSString *identityType = param[@"identityType"];
    NSString *identityValue = param[@"identityValue"];
    
#ifdef DEBUG
    NSLog(@"macroswang>>>>>>>setIdentity|identityType=%@,identityValue=%@",identityType,identityValue);
#endif
    
    [DMHubSDK setIdentityWithType:identityType value:identityValue];

    [weakSelf wrapperCallBack:result
                      channelName:DMHubSDKSetIdentity
                           object:nil];
}
/**
 * 获取当前设置的客户身份
 * 如果当前还未设置过有效的客户身份，返回 nil
 */
- (void)currentIdentity:(NSDictionary *)param result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;
    DMHubIdentity *dmHubIdentity = [DMHubSDK currentIdentity];
    
    [weakSelf wrapperCallBack:result
                      channelName:DMHubSDKCurrentIdentity
                       object:@{ @"type": (dmHubIdentity!=nil)?dmHubIdentity.type:@"",@"value": (dmHubIdentity!=nil)?dmHubIdentity.value:@""}];
}
/**
* 清除当前设置的客户身份

*  当前设置的客户身份，如果当前还未设置过有效的客户身份，返回 nil
*/
- (void)clearIdentity:(NSDictionary *)param result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;
    DMHubIdentity *dmHubIdentity = [DMHubSDK clearIdentity];

    [weakSelf wrapperCallBack:result
                      channelName:DMHubSDKClearIdentity
                       object:@{ @"type": (dmHubIdentity!=nil)?dmHubIdentity.type:@"",@"value": (dmHubIdentity!=nil)?dmHubIdentity.value:@""}];
}
/**
 *跟踪客户自定义事件
 *param中包含 eventId 在 DM Hub 中新建的自定义事件的事件 Id
 *param中包含 properties 事件的自定义属性，必须以在 DM Hub 中新建自定义事件时添加的自定义属性作为 key
 */
- (void)trackMap:(NSDictionary *)param result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;
    //
    [DMHubSDK trackWithEventId:param[@"eventId"]
                    properties:param[@"properties"]];

    [weakSelf wrapperCallBack:result
                      channelName:DMHubSDKTrackMap
                           object:nil];
}
/**
* 跟踪进入视图事件
* param中包含 viewName 视图的名称，客户时间轴上的显示为：'进入手机视图 ${viewName}'
**/
- (void)trackOpenView:(NSDictionary *)param result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;
    //
    [DMHubSDK trackOpenView:param[@"viewName"]];

    [weakSelf wrapperCallBack:result
                      channelName:DMHubSDKTrackOpenView
                           object:nil];
}
/**
* 跟踪离开视图事件
* param包含 viewName 视图的名称，客户时间轴上的显示为：'离开手机视图 ${viewName}'
**/
- (void)trackExitView:(NSDictionary *)param result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;
    //
    [DMHubSDK trackExitView:param[@"viewName"]];

    [weakSelf wrapperCallBack:result
                      channelName:DMHubSDKTrackExitView
                           object:nil];
}
/**
* 设置消息推送的相关信息，以便通过 DM Hub 对客户进行 App 消息推送
*
* param中包含 appKey 在推送服务商注册应用时获得的 AppKey
* param中包含 provider 推送服务商，目前支持极光（@"jpush"）和个推（@"getui"）
* */
- (void)setAppKey:(NSDictionary *)param result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;
    //
    [DMHubSDK setAppKey:param[@"appKey"] forProvider:param[@"provider"]];

    [weakSelf wrapperCallBack:result
                      channelName:DMHubSDKAppKey
                           object:nil];
}
/**
 * 设置消息推送的相关信息，以便通过 DM Hub 对客户进行 App 消息推送
 * param中包含appKey 在推送服务商注册应用时获得的 AppKey
 * param中包含pushId 推送服务商分配给设备的标识，极光的 registrationID 或个推的 clientId
 * param中包含provider 推送服务商，目前支持极光（@"jpush"）和个推（@"getui"）
 */
- (void)setAppKeyWithPushId:(NSDictionary *)param result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;
    //
    [DMHubSDK setAppKey:param[@"appKey"] pushId:param[@"pushId"] forProvider:param[@"provider"]];

    [weakSelf wrapperCallBack:result
                  channelName:DMHubSDKAppKeyWithPushId
                       object:nil];
}

/**
 * 设置消息推送的appKey和clientId 相关信息，以便通过 DM Hub 对客户进行 App 消息推送
 * param中包含appKey 在推送服务商注册应用时获得的 AppKey
 * param中包含clientId 推送服务商分配给设备的标识，极光的 clientId
 */
- (void)onJPushReceiveRegisterId:(NSDictionary *)param result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;
    //
    [DMHubSDK onJPushReceiveRegisterId:param[@"appKey"] withClientId:param[@"clientId"]];
    [weakSelf wrapperCallBack:result
                  channelName:DMHubSDKOnJPushReceiveRegisterId
                       object:nil];
}

/**
 * 跟踪收到 JPush 自定义消息事件
 * @param userInfo  JPush 收到自定义消息的回调方法中传入的 notification
 */
- (void)trackReceiveJPushMessage:(NSDictionary *)userInfo result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;
    NSNotification *notification = [[NSNotification alloc]initWithName:@"" object:nil userInfo:userInfo];
    [DMHubSDK trackReceiveJPushMessage:notification];

    [weakSelf wrapperCallBack:result
                  channelName:DMHubSDKTrackReceiveJPushMessage
                       object:nil];
}

/**
 * 跟踪收到 JPush 自定义消息事件
 * @param bundle  JPush 收到自定义消息的回调方法中传入的
 */
- (void)onJPushReceiveMessageData:(NSDictionary *)bundle result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;
    [DMHubSDK onJPushReceiveMessageData:bundle];

    [weakSelf wrapperCallBack:result
                  channelName:DMHubSDKOnJPushReceiveMessageData
                       object:nil];
}

/**
 * 跟踪收到 JPush 自定义消息事件
 * @param bundle  JPush 收到自定义消息的回调方法中传入的
 */
- (void)onJPushNotificationMessageArrived:(NSDictionary *)bundle result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;
    [DMHubSDK onJPushNotificationMessageArrived:bundle];

    [weakSelf wrapperCallBack:result
                  channelName:DMHubSDKOnJPushNotificationMessageArrived
                       object:nil];
}

/**
 * 设置GETUI消息推送的appKey和clientId 相关信息，以便通过 DM Hub 对客户进行 App 消息推送
 * param中包含appKey 在推送服务商注册应用时获得的 AppKey
 * param中包含clientId 推送服务商分配给设备的标识，GeTui的 clientId
 */
- (void)onGeTuiReceiveClientId:(NSDictionary *)param result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;
    //
    [DMHubSDK onGeTuiReceiveClientId:param[@"appKey"] withClientId:param[@"clientId"]];
    [weakSelf wrapperCallBack:result
                  channelName:DMHubSDKOnJPushReceiveRegisterId
                       object:nil];
}

/**
 * 跟踪收到 GeTui 透传消息事件
 * param中包含 payloadDic GeTui 收到透传消息的回调方法中传入的 payloadData
 * param中包含 offLine GeTui 收到透传消息的回调方法中传入的 offLine(这儿的offLine请在payloadDic中去取)
 */
- (void)trackReceiveGeTuiPayloadData:(NSDictionary *)payloadDic result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;
    //
    NSData *data = [NSJSONSerialization dataWithJSONObject:payloadDic options:NSJSONWritingPrettyPrinted error:nil];
    [DMHubSDK trackReceiveGeTuiPayloadData:data offLine:YES];//这儿的offLine请在payloadDic中去取

    [weakSelf wrapperCallBack:result
                  channelName:DMHubSDKTrackReceiveGeTuiPayloadData
                       object:nil];
}

/**
* 跟踪收到 GeTui 透传消息事件
*
* param中包含 payloadData GeTui 收到透传消息的回调方法中传入的 payloadData
* param中包含 offLine GeTui 收到透传消息的回调方法中传入的 offLine
**/
- (void)onGlobalParamsPush:(NSDictionary *)param result:(FlutterResult)result {
    __weak typeof(self)weakSelf = self;

    [DMHubSDK push:param];

    [weakSelf wrapperCallBack:result
                  channelName:DMHubSDKOnGlobalParamsPush
                       object:nil];
}

@end
